//
//  DudelViewController.m
//  Dudel
//
//  Created by Marcus Wyatt on 14/12/10.
//  Copyright 2010 Exceptionz Software Services Ltd. All rights reserved.
//

#import "DudelViewController.h"

#import "DudelView.h"
#import "PencilTool.h"
#import "LineTool.h"
#import "RectangleTool.h"
#import "EllipseTool.h"
#import "FreehandTool.h"

@implementation DudelViewController

#pragma mark -
#pragma mark Properties

@synthesize dudelView;
@synthesize freehandButton;
@synthesize ellipseButton;
@synthesize rectangleButton;
@synthesize lineButton;
@synthesize pencilButton;
@synthesize currentTool;
@synthesize strokeColor;
@synthesize fillColor;
@synthesize strokeWidth;

/*
// The designated initializer. Override to perform setup that is required before the view is loaded.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}
*/

/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
}
*/



// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
    self.currentTool = [PencilTool sharedPencilTool];
    self.fillColor = [UIColor lightGrayColor];
    self.strokeColor = [UIColor blackColor];
    self.strokeWidth = 2.0;
}


// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return YES;
}

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];

	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
    [super viewDidUnload];

	// Release any retained subviews of the main view.
    self.freehandButton = nil;
    self.ellipseButton = nil;
    self.rectangleButton = nil;
    self.lineButton = nil;
    self.pencilButton = nil;
}


- (void)dealloc {
    [dudelView release], dudelView = nil;
    [currentTool release], currentTool = nil;
    [strokeColor release], strokeColor = nil;
    [fillColor release], fillColor = nil;

    [super dealloc];
}

- (void)deselectAllToolButtons {
//    [[self textButton] setImage:[UIImage imageNamed:@"button_text.png"]];
    [[self freehandButton] setImage:[UIImage imageNamed:@"button_bezier.png"]];
    [[self ellipseButton] setImage:[UIImage imageNamed:@"button_ellipse.png"]];
    [[self rectangleButton] setImage:[UIImage imageNamed:@"button_rectangle.png"]];
    [[self lineButton] setImage:[UIImage imageNamed:@"button_line.png"]];
    [[self pencilButton] setImage:[UIImage imageNamed:@"button_cdots.png"]];
}

- (void)setCurrentTool:(id <Tool>)t {
    [[self currentTool] deactivate];
    if (t != [self currentTool]) {
        [currentTool release];
        currentTool = [t retain];
        [[self currentTool] setDelegate:self];
        [self deselectAllToolButtons];
    }
    [[self currentTool] activate];
    [[self dudelView] setNeedsDisplay];
}

- (IBAction)touchFreehandItem:(id)sender {
    self.currentTool = [FreehandTool sharedFreehandTool];
    [freehandButton setImage:[UIImage imageNamed:@"button_bezier_selected.png"]];
}

- (IBAction)touchEllipseItem:(id)sender {
    [self setCurrentTool:[EllipseTool sharedEllipseTool]];
    [ellipseButton setImage:[UIImage imageNamed:@"button_ellipse_selected.png"]];
}

- (IBAction)touchRectangleItem:(id)sender {
    [self setCurrentTool:[RectangleTool sharedRectangleTool]];
    [rectangleButton setImage:[UIImage imageNamed:@"button_rectangle_selected.png"]];
}

- (IBAction)touchLineItem:(id)sender {
    [self setCurrentTool:[LineTool sharedLineTool]];
    [lineButton setImage:[UIImage imageNamed:@"button_line_selected.png"]];

}

- (IBAction)touchSendPdfEmailItem:(id)sender {

    // Setup the PDF rendering context
    NSMutableData *pdfData = [NSMutableData data];
    UIGraphicsBeginPDFContextToData(pdfData, dudelView.bounds, nil);
    UIGraphicsBeginPDFPage();

    // tell our view to draw
    [dudelView drawRect:dudelView.bounds];

    // remove PDF redering context
    UIGraphicsEndPDFContext();

    // send PDF data in Email message
    MFMailComposeViewController *mailComposer = [[[MFMailComposeViewController alloc] init] autorelease];
    [mailComposer setMailComposeDelegate:self];
    [mailComposer addAttachmentData:pdfData mimeType:@"application/pdf" fileName:@"Dudel creation.pdf"];
    [self presentModalViewController:mailComposer animated:YES];
}


#pragma mark -
#pragma mark MFMailComposeViewControllerDelegate Callbacks

- (void)mailComposeController:(MFMailComposeViewController *)controller didFinishWithResult:(MFMailComposeResult)result error:(NSError *)error {
    [self dismissModalViewControllerAnimated:YES];
}

- (IBAction)touchPencilItem:(id)sender {
    [self setCurrentTool:[PencilTool sharedPencilTool]];
    [[self pencilButton] setImage:[UIImage imageNamed:@"button_cdots_selected.png"]];
}

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
    [[self currentTool] touchesBegan:touches withEvent:event];
    [[self dudelView] setNeedsDisplay];
}

- (void)touchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event {
    [[self currentTool] touchesCancelled:touches withEvent:event];
    [[self dudelView] setNeedsDisplay];
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
    [[self currentTool] touchesEnded:touches withEvent:event];
    [[self dudelView] setNeedsDisplay];
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
    [[self currentTool] touchesMoved:touches withEvent:event];
    [[self dudelView] setNeedsDisplay];
}

- (void)addDrawable:(id<Drawable>) d {
    [[[self dudelView] drawables] addObject:d];
    [[self dudelView] setNeedsDisplay];
}

- (UIView *)viewForUseWithTool:(id <Tool>)t {
    return self.view;
}

- (void)drawTemporary {
    [[self currentTool] drawTemporary];
}

@end
