//
//  RectangleTool.m
//  Dudel
//
//  Created by Marcus Wyatt on 15/12/10.
//  Copyright 2010 Exceptionz Software Services Ltd. All rights reserved.
//

#import "RectangleTool.h"
#import "PathDrawingInfo.h"
#import "SynthesizeSingleton.h"

@implementation RectangleTool

@synthesize delegate;

SYNTHESIZE_SINGLETON_FOR_CLASS(RectangleTool);

- init {
    if ((self = [super init])) {
        trackingTouches = [[NSMutableArray array] retain];
        startPoints = [[NSMutableArray array] retain];
    }
    return self;
}

- (void)activate {

}

- (void)deactivate {
    [trackingTouches removeAllObjects];
    [startPoints removeAllObjects];
}

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
    UIView *touchedView = [delegate viewForUseWithTool:self];

    for (UITouch *touch in [event allTouches]) {
        [trackingTouches addObject:touch];
        CGPoint location = [touch locationInView:touchedView];
        [startPoints addObject:[NSValue valueWithCGPoint:location]];
    }
}

- (void)touchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event {

}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
    UIView *touchedView = [delegate viewForUseWithTool:self];

    for (UITouch *touch in [event allTouches]) {
        NSUInteger touchIndex = [trackingTouches indexOfObject:touch];
        if (touchIndex != NSNotFound) {
            CGPoint startPoint = [[startPoints objectAtIndex:touchIndex] CGPointValue];
            CGPoint endPoint = [touch locationInView:touchedView];
            CGRect rect = CGRectMake(startPoint.x, startPoint.y, endPoint.x - startPoint.x, endPoint.y - startPoint.y);
            UIBezierPath *path = [UIBezierPath bezierPathWithRect:rect];
            PathDrawingInfo *info = [PathDrawingInfo pathDrawingInfoWithPath:path fillColor:delegate.fillColor strokeColor:delegate.strokeColor];
            [delegate addDrawable:info];
            [trackingTouches removeObjectAtIndex:touchIndex];
            [startPoints removeObjectAtIndex:touchIndex];
        }
    }
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {

}

- (void)drawTemporary {
    UIView *touchedView = [delegate viewForUseWithTool:self];

    for (int i = 0; i < [trackingTouches count]; i++) {
        UITouch *touch = [trackingTouches objectAtIndex:i];
        CGPoint startPoint = [[startPoints objectAtIndex:i] CGPointValue];
        CGPoint endPoint = [touch locationInView:touchedView];
        CGRect rect = CGRectMake(startPoint.x, startPoint.y, endPoint.x - startPoint.x, endPoint.y - startPoint.y);
        UIBezierPath *path = [UIBezierPath bezierPathWithRect:rect];
        [delegate.fillColor setFill];
        [path fill];
        [delegate.strokeColor setStroke];
        [path stroke];
    }
}

- (void)dealloc {
    [trackingTouches release], trackingTouches = nil;
    [startPoints release], startPoints = nil;
    self.delegate = nil;
    [super dealloc];
}
@end
